==> robomotion_app_lib-0.1.3.tar.gz: lib/robomotion_app_lib.dart <==
library robomotion_app_lib;

import 'dart:async';
import 'dart:convert';
import 'dart:typed_data';

import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';
import 'package:dio/dio.dart';
import 'package:robomotion_app_lib/encryption.dart';
import 'package:robomotion_app_lib/models/published_flow.dart';
import 'package:robomotion_app_lib/models/queue_message.dart';
import 'package:web_socket_channel/web_socket_channel.dart';
import 'package:uuid/uuid.dart';

Dio _dio = Dio();
StreamController<void> _keyExStreamCtrl = StreamController<void>();
StreamController<void> _robotIDStreamCtrl = StreamController<void>();

class Robomotion {
  final Config config;
  final Widget app;
  final Function(QueueMessage data) onMsgRecv;
  String _apiURL;
  String _appURL;
  WebSocketChannel _channel;

  Robomotion({@required this.config, @required this.app, @required this.onMsgRecv}) {
    _apiURL = _getAPIUrl();
    _appURL = _getAppUrl();
    generateAESKey();
  }

  Future<void> run() async {
    await _connectAppServer();
    runApp(this.app);
  }

  Future<List<PublishedFlow>> getPublishedFlows() async {
    String url = "${this._apiURL}/v1/flows.published.list";
    Map resp = await _request(method: "GET", url: url, queryParams: {"page": 1, "size": 100, "search": ""});

    List<PublishedFlow> flows = [];
    for (dynamic f in resp["flows"]) {
      flows.add(PublishedFlow.fromJson(f));
    }
    return flows;
  }

  Future<void> runFlow(String robotID, String flowID) async {
    String url = "${this._apiURL}/v1/apps.flow.run";
    final resp =
        await _request(method: "POST", url: url, body: {"robot_id": robotID, "app_id": config.id, "flow_id": flowID});
    config.robotID = resp.containsKey("robot_id") ? resp["robot_id"] : "";
  }

  Future<Map> _request({@required String method, @required url, Map body, Map<String, dynamic> queryParams}) async {
    final resp = await _dio.request(url,
        queryParameters: queryParams,
        options: Options(method: method, headers: {
          'Authorization': 'Bearer ${this.config.token}',
          'Content-Type': 'application/json; charset=UTF-8',
        }),
        data: body);

    if (!resp.data.containsKey("ok") || !resp.data["ok"]) {
      throw new Exception("Request failed: ${resp.data.toString()}");
    }

    return resp.data;
  }

  Future<void> _connectAppServer() async {
    String url = "$_appURL?id=${config.id}&sid=${config.token}";
    _channel = WebSocketChannel.connect(Uri.parse(url));
    _channel.stream.listen((data) {
      final msg = QueueMessage.fromJson(jsonDecode(data));
      if (msg.encrypted) {
        msg.payload = decryptPayload(msg.payload);
      }
      _onMsgRecv(msg).then((_) {
        if (onMsgRecv != null) onMsgRecv(msg);
      });
    });
  }

  Future<void> _onMsgRecv(QueueMessage msg) async {
    if (msg.msgType == QMT_Error) {
      return;
    }

    if (config.robotID == "" && msg.msgType == QMT_RobotID) {
      await _robotIDStreamCtrl.stream.first;
    }

    switch (msg.msgType) {
      case QMT_AppRequest:
        final resp = QueueMessage(
            id: msg.id,
            appID: config.id,
            robotID: config.robotID,
            msgType: QMT_AppRequest,
            payload: utf8.encode('{"msg": "received"}'),
            encrypted: true);

        sendAppMsg(resp);
        break;

      case QMT_KeyExchange:
        final pem = msg.payload;
        robotPubKeyPem = utf8.decode(pem);
        robotPubKey = parsePublicKey(pem);
        _keyExStreamCtrl.add(null);
        break;

      case QMT_RobotID:
        config.robotID = utf8.decode(msg.payload);
        _robotIDStreamCtrl.add(null);
        break;
    }
  }

  void sendAppMsg(QueueMessage msg) {
    if (msg.encrypted) {
      msg.payload = encryptPayload(msg.payload);
    }
    _channel.sink.add(msg.toString());
  }

  Future<void> keyExchange() async {
    var msg = QueueMessage(
      id: Uuid().v4(),
      appID: config.id,
      robotID: config.robotID,
      userID: config.userID,
      msgType: QMT_KeyExchange,
      payload: null,
      encrypted: false,
    );

    sendAppMsg(msg);
    await _keyExStreamCtrl.stream.first;

    final encAes = rsaEncrypt(robotPubKey, aesKey.bytes);
    msg = QueueMessage(
        id: Uuid().v4(),
        appID: config.id,
        robotID: config.robotID,
        msgType: QMT_KeyExchange,
        payload: encAes,
        encrypted: false);

    sendAppMsg(msg);
  }

  String appIn(Map payload, {appID, robotID}) {
    var msg = QueueMessage(
      id: Uuid().v4(),
      appID: appID == null ? config.id : appID,
      robotID: robotID == null ? config.robotID : robotID,
      userID: config.userID,
      msgType: QMT_AppIn,
      payload: utf8.encode(jsonEncode({"payload": payload})),
      encrypted: true,
    );

    sendAppMsg(msg);
    return msg.id;
  }

  Future<String> _getUploadLink(String file) async {
    String url = "${this._apiURL}/v1/apps.upload.get";
    var resp = await _request(method: "GET", url: url, queryParams: {"app_id": config.id, "file": file});
    if (!resp.containsKey("url")) {
      throw Exception(resp.toString());
    }
    return Uri.decodeFull(resp["url"]);
  }

  Future<void> uploadFile(String file, Uint8List data) async {
    String url = await _getUploadLink(file);
    data = encryptPayload(data);
    await _dio.put(url,
        data: base64Encode(data), options: Options(headers: {"Content-Type": "application/octet-stream"}));
  }

  Future<String> getFileURL(String file) async {
    String url = "${this._apiURL}/v1/apps.file.get";
    var resp = await _request(method: "GET", url: url, queryParams: {"app_id": config.id, "file": file});
    if (!resp.containsKey("url")) {
      throw Exception(resp.toString());
    }
    return resp["url"];
  }

  String _getAPIUrl() {
    String ws = this.config.workspace;
    List<String> parts = ws.split(".");
    String domain = parts.getRange(1, parts.length).join(".");
    String protocol = parts[0].split("//")[0];
    return domain == "robomotion.io" ? "$protocol//api.$domain" : "$ws/api";
  }

  String _getAppUrl() {
    String ws = this.config.workspace;
    List<String> parts = ws.split(".");
    String domain = parts.getRange(1, parts.length).join(".");
    String protocol = parts[0].split("//")[0] == "https:" ? "wss:" : "ws:";
    return domain == "robomotion.io"
        ? "$protocol//apps.$domain"
        : "$protocol//${parts[0].split("//")[1]}.$domain/app/app";
  }
}

class Config {
  final String id;
  final String namespace;
  final String token;
  final String userID;
  final String repositoryID;
  final String workspace;
  String robotID;
  String flowID;

  Config(
      {@required this.id,
      @required this.namespace,
      @required this.token,
      @required this.userID,
      @required this.repositoryID,
      @required this.workspace});
}

==> robomotion_app_lib-0.1.4.tar.gz: lib/robomotion_app_lib.dart <==
library robomotion_app_lib;

import 'dart:async';
import 'dart:convert';
import 'dart:typed_data';

import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';
import 'package:dio/dio.dart';
import 'package:robomotion_app_lib/encryption.dart';
import 'package:robomotion_app_lib/models/published_flow.dart';
import 'package:robomotion_app_lib/models/queue_message.dart';
import 'package:web_socket_channel/web_socket_channel.dart';
import 'package:uuid/uuid.dart';

Dio _dio = Dio();
StreamController<void> _keyExStreamCtrl = StreamController<void>();
StreamController<void> _robotIDStreamCtrl = StreamController<void>();

class Robomotion {
  final Config config;
  final Widget app;
  final Function(QueueMessage data) onMsgRecv;
  String _apiURL;
  String _appURL;
  WebSocketChannel _channel;

  Robomotion({@required this.config, @required this.app, @required this.onMsgRecv}) {
    _apiURL = _getAPIUrl();
    _appURL = _getAppUrl();
    generateAESKey();
  }

  Future<void> run() async {
    await _connectAppServer();
    runApp(this.app);
  }

  Future<List<PublishedFlow>> getPublishedFlows() async {
    String url = "${this._apiURL}/v1/flows.published.list";
    Map resp = await _request(method: "GET", url: url, queryParams: {"page": 1, "size": 100, "search": ""});

    List<PublishedFlow> flows = [];
    for (dynamic f in resp["flows"]) {
      flows.add(PublishedFlow.fromJson(f));
    }
    return flows;
  }

  Future<void> runFlow(String robotID, String flowID) async {
    String url = "${this._apiURL}/v1/apps.flow.run";
    final resp =
        await _request(method: "POST", url: url, body: {"robot_id": robotID, "app_id": config.id, "flow_id": flowID});
    config.robotID = resp.containsKey("robot_id") ? resp["robot_id"] : "";
  }

  Future<Map> _request({@required String method, @required url, Map body, Map<String, dynamic> queryParams}) async {
    final resp = await _dio.request(url,
        queryParameters: queryParams,
        options: Options(method: method, headers: {
          'Authorization': 'Bearer ${this.config.token}',
          'Content-Type': 'application/json; charset=UTF-8',
        }),
        data: body);

    if (!resp.data.containsKey("ok") || !resp.data["ok"]) {
      throw new Exception("Request failed: ${resp.data.toString()}");
    }

    return resp.data;
  }

  Future<void> _connectAppServer() async {
    String url = "$_appURL?id=${config.id}&sid=${config.token}";
    _channel = WebSocketChannel.connect(Uri.parse(url));
    _channel.stream.listen((data) {
      final msg = QueueMessage.fromJson(jsonDecode(data));
      if (msg.encrypted) {
        msg.payload = decryptPayload(msg.payload);
      }
      _onMsgRecv(msg).then((_) {
        if (onMsgRecv != null) onMsgRecv(msg);
      });
    });
  }

  Future<void> _onMsgRecv(QueueMessage msg) async {
    if (msg.msgType == QMT_Error) {
      return;
    }

    if (config.robotID == "" && msg.msgType == QMT_RobotID) {
      await _robotIDStreamCtrl.stream.first;
    }

    switch (msg.msgType) {
      case QMT_AppRequest:
        final resp = QueueMessage(
            id: msg.id,
            appID: config.id,
            robotID: config.robotID,
            msgType: QMT_AppRequest,
            payload: utf8.encode('{"msg": "received"}'),
            encrypted: true);

        sendAppMsg(resp);
        break;

      case QMT_KeyExchange:
        final pem = msg.payload;
        robotPubKeyPem = utf8.decode(pem);
        robotPubKey = parsePublicKey(pem);
        _keyExStreamCtrl.add(null);
        break;

      case QMT_RobotID:
        config.robotID = utf8.decode(msg.payload);
        _robotIDStreamCtrl.add(null);
        break;
    }
  }

  void sendAppMsg(QueueMessage msg) {
    if (msg.encrypted) {
      msg.payload = encryptPayload(msg.payload);
    }
    _channel.sink.add(msg.toString());
  }

  Future<void> keyExchange() async {
    var msg = QueueMessage(
      id: Uuid().v4(),
      appID: config.id,
      robotID: config.robotID,
      userID: config.userID,
      msgType: QMT_KeyExchange,
      payload: null,
      encrypted: false,
    );

    sendAppMsg(msg);
    await _keyExStreamCtrl.stream.first;

    final encAes = rsaEncrypt(robotPubKey, aesKey.bytes);
    msg = QueueMessage(
        id: Uuid().v4(),
        appID: config.id,
        robotID: config.robotID,
        msgType: QMT_KeyExchange,
        payload: encAes,
        encrypted: false);

    sendAppMsg(msg);
  }

  String appIn(Map payload, {appID, robotID}) {
    var msg = QueueMessage(
      id: Uuid().v4(),
      appID: appID == null ? config.id : appID,
      robotID: robotID == null ? config.robotID : robotID,
      userID: config.userID,
      msgType: QMT_AppIn,
      payload: utf8.encode(jsonEncode({"payload": payload})),
      encrypted: true,
    );

    sendAppMsg(msg);
    return msg.id;
  }

  Future<String> _getUploadLink(String file) async {
    String url = "${this._apiURL}/v1/apps.upload.get";
    var resp = await _request(method: "GET", url: url, queryParams: {"app_id": config.id, "file": file});
    if (!resp.containsKey("url")) {
      throw Exception(resp.toString());
    }
    return Uri.decodeFull(resp["url"]);
  }

  Future<void> uploadFile(String file, Uint8List data) async {
    String url = await _getUploadLink(file);
    data = encryptPayload(data);
    await _dio.put(url, data: data.toList(), options: Options(headers: {"Content-Type": "application/octet-stream"}));
  }

  Future<String> getFileURL(String file) async {
    String url = "${this._apiURL}/v1/apps.file.get";
    var resp = await _request(method: "GET", url: url, queryParams: {"app_id": config.id, "file": file});
    if (!resp.containsKey("url")) {
      throw Exception(resp.toString());
    }
    return resp["url"];
  }

  String _getAPIUrl() {
    String ws = this.config.workspace;
    List<String> parts = ws.split(".");
    String domain = parts.getRange(1, parts.length).join(".");
    String protocol = parts[0].split("//")[0];
    return domain == "robomotion.io" ? "$protocol//api.$domain" : "$ws/api";
  }

  String _getAppUrl() {
    String ws = this.config.workspace;
    List<String> parts = ws.split(".");
    String domain = parts.getRange(1, parts.length).join(".");
    String protocol = parts[0].split("//")[0] == "https:" ? "wss:" : "ws:";
    return domain == "robomotion.io"
        ? "$protocol//apps.$domain"
        : "$protocol//${parts[0].split("//")[1]}.$domain/app/app";
  }
}

class Config {
  final String id;
  final String namespace;
  final String token;
  final String userID;
  final String repositoryID;
  final String workspace;
  String robotID;
  String flowID;

  Config(
      {@required this.id,
      @required this.namespace,
      @required this.token,
      @required this.userID,
      @required this.repositoryID,
      @required this.workspace});
}
